//Microsoft Developer Studio generated resource script.
//
#include "resource.h"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
#include "afxres.h"

/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// English (U.S.) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)
#ifdef _WIN32
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US
#pragma code_page(1252)
#endif //_WIN32

#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE DISCARDABLE 
BEGIN
    "resource.h\0"
END

2 TEXTINCLUDE DISCARDABLE 
BEGIN
    "#include ""afxres.h""\r\n"
    "\0"
END

3 TEXTINCLUDE DISCARDABLE 
BEGIN
    "\r\n"
    "\0"
END

#endif    // APSTUDIO_INVOKED


#ifndef _MAC
/////////////////////////////////////////////////////////////////////////////
//
// Version
//

VS_VERSION_INFO VERSIONINFO
 FILEVERSION 1,0,1,1
 PRODUCTVERSION 1,0,1,1
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x40004L
 FILETYPE 0x2L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904e4"
        BEGIN
            VALUE "Comments", "\0"
            VALUE "CompanyName", "Jason Summers\0"
            VALUE "FileDescription", "MNG Plug-in 1.0.1\0"
            VALUE "FileExtents", "mng|mng|jng|jng\0"
            VALUE "FileOpenName", "MNG Animation (*.mng)|MNG Animation (*.mng)|JNG Image (*.jng)|JNG Image (*.jng)\0"
            VALUE "FileVersion", "1.0.1\0"
            VALUE "InternalName", "npmngplg\0"
            VALUE "LegalCopyright", "Copyright © 2000-2002\0"
            VALUE "LegalTrademarks", "\0"
            VALUE "MIMEType", "video/mng|video/x-mng|image/jng|image/x-jng\0"
            VALUE "OriginalFilename", "npmngplg.dll\0"
            VALUE "PrivateBuild", "\0"
            VALUE "ProductName", "MNG Plug-in 1.0.1\0"
            VALUE "ProductVersion", "1.0.1\0"
            VALUE "SpecialBuild", "\0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1252
    END
END

#endif    // !_MAC


/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

ABOUTDLG DIALOG DISCARDABLE  0, 0, 282, 137
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "MNGPLG"
FONT 8, "MS Sans Serif"
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,225,7,50,14,WS_GROUP
    EDITTEXT        IDC_PRGINFO,7,7,213,123,ES_MULTILINE | ES_AUTOVSCROLL | 
                    ES_READONLY | NOT WS_BORDER | WS_VSCROLL | WS_GROUP
END

PROPDLG DIALOG DISCARDABLE  0, 0, 344, 186
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "Image properties"
FONT 8, "MS Sans Serif"
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,287,7,50,14,WS_GROUP
    EDITTEXT        IDC_IMGINFO,7,7,275,97,ES_MULTILINE | ES_AUTOVSCROLL | 
                    ES_AUTOHSCROLL | ES_READONLY | WS_VSCROLL | WS_GROUP
    LTEXT           "Image comments:",IDC_STATIC,7,108,56,8
    EDITTEXT        IDC_MNGTEXT,7,119,275,60,ES_MULTILINE | ES_READONLY | 
                    WS_VSCROLL | WS_GROUP
END


/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO DISCARDABLE 
BEGIN
    "ABOUTDLG", DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 275
        TOPMARGIN, 7
        BOTTOMMARGIN, 130
    END

    "PROPDLG", DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 337
        TOPMARGIN, 7
        BOTTOMMARGIN, 179
    END
END
#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Cursor
//

CURHAND_NS              CURSOR  DISCARDABLE     "cur_ns.cur"
#endif    // English (U.S.) resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//


/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

