//{{NO_DEPENDENCIES}}
// Microsoft Developer Studio generated include file.
// Used by npmngplg.rc
//
#define IDR_DATA1                       106
#define IDC_PRGINFO                     1000
#define IDC_IMGINFO                     1001
#define IDC_MNGTEXT                     1002

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        110
#define _APS_NEXT_COMMAND_VALUE         40001
#define _APS_NEXT_CONTROL_VALUE         1004
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
