/* ************************************************************************** */
/* *                                                                        * */
/* * COPYRIGHT NOTICE:                                                      * */
/* *                                                                        * */
/* * Copyright (c) 2000-2007 Gerard Juyn (gerard@libmng.com)                * */
/* * [You may insert additional notices after this sentence if you modify   * */
/* *  this source]                                                          * */
/* *                                                                        * */
/* * For the purposes of this copyright and license, "Contributing Authors" * */
/* * is defined as the following set of individuals:                        * */
/* *                                                                        * */
/* *    Gerard Juyn                                                         * */
/* *    (hopefully some more to come...)                                    * */
/* *                                                                        * */
/* * The MNG Library is supplied "AS IS".  The Contributing Authors         * */
/* * disclaim all warranties, expressed or implied, including, without      * */
/* * limitation, the warranties of merchantability and of fitness for any   * */
/* * purpose.  The Contributing Authors assume no liability for direct,     * */
/* * indirect, incidental, special, exemplary, or consequential damages,    * */
/* * which may result from the use of the MNG Library, even if advised of   * */
/* * the possibility of such damage.                                        * */
/* *                                                                        * */
/* * Permission is hereby granted to use, copy, modify, and distribute this * */
/* * source code, or portions hereof, for any purpose, without fee, subject * */
/* * to the following restrictions:                                         * */
/* *                                                                        * */
/* * 1. The origin of this source code must not be misrepresented;          * */
/* *    you must not claim that you wrote the original software.            * */
/* *                                                                        * */
/* * 2. Altered versions must be plainly marked as such and must not be     * */
/* *    misrepresented as being the original source.                        * */
/* *                                                                        * */
/* * 3. This Copyright notice may not be removed or altered from any source * */
/* *    or altered source distribution.                                     * */
/* *                                                                        * */
/* * The Contributing Authors specifically permit, without fee, and         * */
/* * encourage the use of this source code as a component to supporting     * */
/* * the MNG and JNG file format in commercial products.  If you use this   * */
/* * source code in a product, acknowledgment would be highly appreciated.  * */
/* *                                                                        * */
/* ************************************************************************** */
/* *                                                                        * */
/* * project   : libmng                                                     * */
/* * file      : libmng.cpp                copyright (c) 2000-2002 G.Juyn   * */
/* * version   : 1.0.5                                                      * */
/* *                                                                        * */
/* * purpose   : generic dll project assembly file                          * */
/* *                                                                        * */
/* * author    : G.Juyn                                                     * */
/* * web       : http://www.3-t.com                                         * */
/* * email     : mailto:info@3-t.com                                        * */
/* *                                                                        * */
/* * comment   : Autogenerated file with the libmng.dll BCB project         * */
/* *                                                                        * */
/* * changes   : 0.5.1 - 05/14/2000 - G.Juyn                                * */
/* *             - added this block                                         * */
/* *                                                                        * */
/* *             1.0.5 - 08/20/2000 - G.Juyn                                * */
/* *             - version-number & copyright                               * */
/* *                                                                        * */
/* ************************************************************************** */

#include <windows.h>
#pragma hdrstop
#include <condefs.h>

/* ************************************************************************** */

USERES("libmng.res");
USEUNIT("..\..\libmng_hlapi.c");
USEUNIT("..\..\libmng_callback_xs.c");
USEUNIT("..\..\libmng_prop_xs.c");
USEUNIT("..\..\libmng_chunk_xs.c");
USEUNIT("..\..\libmng_object_prc.c");
USEUNIT("..\..\libmng_chunk_descr.c");
USEUNIT("..\..\libmng_chunk_prc.c");
USEUNIT("..\..\libmng_chunk_io.c");
USEUNIT("..\..\libmng_read.c");
USEUNIT("..\..\libmng_write.c");
USEUNIT("..\..\libmng_display.c");
USEUNIT("..\..\libmng_dither.c");
USEUNIT("..\..\libmng_pixels.c");
USEUNIT("..\..\libmng_filter.c");
USEUNIT("..\..\libmng_error.c");
USEUNIT("..\..\libmng_trace.c");
USEUNIT("..\..\libmng_cms.c");
USEUNIT("..\..\libmng_zlib.c");
USEUNIT("..\..\libmng_jpeg.c");
USEUNIT("..\..\..\zlib\adler32.c");
USEUNIT("..\..\..\zlib\compress.c");
USEUNIT("..\..\..\zlib\crc32.c");
USEUNIT("..\..\..\zlib\deflate.c");
USEUNIT("..\..\..\zlib\inffast.c");
USEUNIT("..\..\..\zlib\inflate.c");
USEUNIT("..\..\..\zlib\inftrees.c");
USEUNIT("..\..\..\zlib\trees.c");
USEUNIT("..\..\..\zlib\uncompr.c");
USEUNIT("..\..\..\zlib\zutil.c");
USEUNIT("..\..\..\jpgsrc6b\jquant2.c");
USEUNIT("..\..\..\jpgsrc6b\jcapistd.c");
USEUNIT("..\..\..\jpgsrc6b\jccoefct.c");
USEUNIT("..\..\..\jpgsrc6b\jccolor.c");
USEUNIT("..\..\..\jpgsrc6b\jcdctmgr.c");
USEUNIT("..\..\..\jpgsrc6b\jchuff.c");
USEUNIT("..\..\..\jpgsrc6b\jcinit.c");
USEUNIT("..\..\..\jpgsrc6b\jcmainct.c");
USEUNIT("..\..\..\jpgsrc6b\jcmarker.c");
USEUNIT("..\..\..\jpgsrc6b\jcmaster.c");
USEUNIT("..\..\..\jpgsrc6b\jcomapi.c");
USEUNIT("..\..\..\jpgsrc6b\jcparam.c");
USEUNIT("..\..\..\jpgsrc6b\jcphuff.c");
USEUNIT("..\..\..\jpgsrc6b\jcprepct.c");
USEUNIT("..\..\..\jpgsrc6b\jcsample.c");
USEUNIT("..\..\..\jpgsrc6b\jctrans.c");
USEUNIT("..\..\..\jpgsrc6b\jdapistd.c");
USEUNIT("..\..\..\jpgsrc6b\jdatadst.c");
USEUNIT("..\..\..\jpgsrc6b\jdatasrc.c");
USEUNIT("..\..\..\jpgsrc6b\jdcoefct.c");
USEUNIT("..\..\..\jpgsrc6b\jdcolor.c");
USEUNIT("..\..\..\jpgsrc6b\jddctmgr.c");
USEUNIT("..\..\..\jpgsrc6b\jdhuff.c");
USEUNIT("..\..\..\jpgsrc6b\jdinput.c");
USEUNIT("..\..\..\jpgsrc6b\jdmainct.c");
USEUNIT("..\..\..\jpgsrc6b\jdmarker.c");
USEUNIT("..\..\..\jpgsrc6b\jdmaster.c");
USEUNIT("..\..\..\jpgsrc6b\jdmerge.c");
USEUNIT("..\..\..\jpgsrc6b\jdphuff.c");
USEUNIT("..\..\..\jpgsrc6b\jdpostct.c");
USEUNIT("..\..\..\jpgsrc6b\jdsample.c");
USEUNIT("..\..\..\jpgsrc6b\jdtrans.c");
USEUNIT("..\..\..\jpgsrc6b\jerror.c");
USEUNIT("..\..\..\jpgsrc6b\jfdctflt.c");
USEUNIT("..\..\..\jpgsrc6b\jfdctfst.c");
USEUNIT("..\..\..\jpgsrc6b\jfdctint.c");
USEUNIT("..\..\..\jpgsrc6b\jidctflt.c");
USEUNIT("..\..\..\jpgsrc6b\jidctfst.c");
USEUNIT("..\..\..\jpgsrc6b\jidctint.c");
USEUNIT("..\..\..\jpgsrc6b\jidctred.c");
USEUNIT("..\..\..\jpgsrc6b\jmemmgr.c");
USEUNIT("..\..\..\jpgsrc6b\jmemnobs.c");
USEUNIT("..\..\..\jpgsrc6b\jquant1.c");
USEUNIT("..\..\..\jpgsrc6b\jcapimin.c");
USEUNIT("..\..\..\jpgsrc6b\jutils.c");
USEUNIT("..\..\..\jpgsrc6b\jdapimin.c");
USELIB("..\..\..\lcms\Projects\Bcc-5.5-static\lcmsstat.lib");
//---------------------------------------------------------------------------
#pragma argsused
int WINAPI DllEntryPoint(HINSTANCE hinst, unsigned long reason, void*)
{
        return 1;
}
/* ************************************************************************** */

