LIBRARY libmng
EXPORTS
	jcopy_block_row
	jcopy_sample_rows
	jdiv_round_up
	jinit_1pass_quantizer
	jinit_2pass_quantizer
	jinit_c_coef_controller
	jinit_c_main_controller
	jinit_c_master_control
	jinit_c_prep_controller
	jinit_color_converter
	jinit_color_deconverter
	jinit_compress_master
	jinit_d_coef_controller
	jinit_d_main_controller
	jinit_d_post_controller
	jinit_downsampler
	jinit_forward_dct
	jinit_huff_decoder
	jinit_huff_encoder
	jinit_input_controller
	jinit_inverse_dct
	jinit_marker_reader
	jinit_marker_writer
	jinit_master_decompress
	jinit_memory_mgr
	jinit_merged_upsampler
	jinit_phuff_decoder
	jinit_phuff_encoder
	jinit_upsampler
	jpeg_CreateCompress
	jpeg_CreateDecompress
	jpeg_abort
	jpeg_abort_compress
	jpeg_abort_decompress
	jpeg_add_quant_table
	jpeg_alloc_huff_table
	jpeg_alloc_quant_table
	jpeg_calc_output_dimensions
	jpeg_consume_input
	jpeg_copy_critical_parameters
	jpeg_default_colorspace
	jpeg_destroy
	jpeg_destroy_compress
	jpeg_destroy_decompress
	jpeg_fdct_float
	jpeg_fdct_ifast
	jpeg_fdct_islow
	jpeg_fill_bit_buffer
	jpeg_finish_compress
	jpeg_finish_decompress
	jpeg_finish_output
	jpeg_free_large
	jpeg_free_small
	jpeg_gen_optimal_table
	jpeg_get_large
	jpeg_get_small
	jpeg_has_multiple_scans
	jpeg_huff_decode
	jpeg_save_markers
	jpeg_set_colorspace
	jpeg_set_defaults
	jpeg_set_linear_quality
	jpeg_set_marker_processor
	jpeg_set_quality
	jpeg_simple_progression
	jpeg_start_compress
	jpeg_start_decompress
	jpeg_start_output
	jpeg_std_error
	jpeg_stdio_dest
	jpeg_stdio_src
	jpeg_suppress_tables
	jpeg_write_coefficients
	jpeg_write_m_byte
	jpeg_write_m_header
	jpeg_write_marker
	jpeg_write_raw_data
	jpeg_write_scanlines
	jpeg_write_tables
	jround_up
	jzero_far
	mng_cleanup
	mng_copy_chunk
	mng_create
	mng_display
	mng_display_freeze
	mng_display_goframe
	mng_display_golayer
	mng_display_gotime
	mng_display_reset
	mng_display_resume
	mng_get_alphabitdepth
	mng_get_alphacompression
	mng_get_alphadepth
	mng_get_alphafilter
	mng_get_alphainterlace
	mng_get_bgcolor
	mng_get_bitdepth
	mng_get_bkgdstyle
	mng_get_cacheplayback
	mng_get_canvasstyle
	mng_get_colortype
	mng_get_compression
	mng_get_currentframe
	mng_get_currentlayer
	mng_get_currentplaytime
	mng_get_dfltimggamma
	mng_get_dfltimggammaint
	mng_get_displaygamma
	mng_get_displaygammaint
	mng_get_doprogressive
	mng_get_filter
	mng_get_framecount
	mng_get_imageheight
	mng_get_imagelevel
	mng_get_imagetype
	mng_get_imagewidth
	mng_get_interlace
	mng_get_jpeg_dctmethod
	mng_get_jpeg_maxjdat
	mng_get_jpeg_optimized
	mng_get_jpeg_progressive
	mng_get_jpeg_quality
	mng_get_jpeg_smoothing
	mng_get_lastbackchunk
	mng_get_lastseekname
	mng_get_layercount
	mng_get_maxcanvasheight
	mng_get_maxcanvaswidth
	mng_get_playtime
	mng_get_refreshpass
	mng_get_runtime
	mng_get_sectionbreaks
	mng_get_sigtype
	mng_get_simplicity
	mng_get_speed
	mng_get_srgb
	mng_get_starttime
	mng_get_storechunks
	mng_get_suspensionmode
	mng_get_ticks
	mng_get_totalframes
	mng_get_totallayers
	mng_get_totalplaytime
	mng_get_usebkgd
	mng_get_userdata
	mng_get_viewgamma
	mng_get_viewgammaint
	mng_get_zlib_level
	mng_get_zlib_maxidat
	mng_get_zlib_memlevel
	mng_get_zlib_method
	mng_get_zlib_strategy
	mng_get_zlib_windowbits
	mng_getcb_closestream
	mng_getcb_errorproc
	mng_getcb_getalphaline
	mng_getcb_getbkgdline
	mng_getcb_getcanvasline
	mng_getcb_gettickcount
	mng_getcb_memalloc
	mng_getcb_memfree
	mng_getcb_openstream
	mng_getcb_processheader
	mng_getcb_processneed
	mng_getcb_processsave
	mng_getcb_processseek
	mng_getcb_processterm
	mng_getcb_processtext
	mng_getcb_processunknown
	mng_getcb_readdata
	mng_getcb_refresh
	mng_getcb_settimer
	mng_getcb_writedata
	mng_getchunk_back
	mng_getchunk_basi
	mng_getchunk_bkgd
	mng_getchunk_chrm
	mng_getchunk_clip
	mng_getchunk_clon
	mng_getchunk_dbyk
	mng_getchunk_defi
	mng_getchunk_dhdr
	mng_getchunk_disc
	mng_getchunk_drop
	mng_getchunk_endl
	mng_getchunk_evnt
	mng_getchunk_evnt_entry
	mng_getchunk_expi
	mng_getchunk_fpri
	mng_getchunk_fram
	mng_getchunk_gama
	mng_getchunk_hist
	mng_getchunk_iccp
	mng_getchunk_idat
	mng_getchunk_ihdr
	mng_getchunk_itxt
	mng_getchunk_jdat
	mng_getchunk_jhdr
	mng_getchunk_loop
	mng_getchunk_magn
	mng_getchunk_mhdr
	mng_getchunk_move
	mng_getchunk_need
	mng_getchunk_ordr
	mng_getchunk_ordr_entry
	mng_getchunk_past
	mng_getchunk_past_src
	mng_getchunk_phyg
	mng_getchunk_phys
	mng_getchunk_plte
	mng_getchunk_pplt
	mng_getchunk_pplt_entry
	mng_getchunk_prom
	mng_getchunk_save
	mng_getchunk_save_entry
	mng_getchunk_sbit
	mng_getchunk_seek
	mng_getchunk_show
	mng_getchunk_splt
	mng_getchunk_srgb
	mng_getchunk_term
	mng_getchunk_text
	mng_getchunk_time
	mng_getchunk_trns
	mng_getchunk_unknown
	mng_getchunk_ztxt
	mng_getimgdata_chunk
	mng_getimgdata_chunkseq
	mng_getimgdata_seq
	mng_getlasterror
	mng_initialize
	mng_iterate_chunks
	mng_putchunk_back
	mng_putchunk_basi
	mng_putchunk_bkgd
	mng_putchunk_chrm
	mng_putchunk_clip
	mng_putchunk_clon
	mng_putchunk_dbyk
	mng_putchunk_defi
	mng_putchunk_dhdr
	mng_putchunk_disc
	mng_putchunk_drop
	mng_putchunk_endl
	mng_putchunk_evnt
	mng_putchunk_evnt_entry
	mng_putchunk_expi
	mng_putchunk_fpri
	mng_putchunk_fram
	mng_putchunk_gama
	mng_putchunk_hist
	mng_putchunk_iccp
	mng_putchunk_idat
	mng_putchunk_iend
	mng_putchunk_ihdr
	mng_putchunk_ipng
	mng_putchunk_itxt
	mng_putchunk_jdat
	mng_putchunk_jhdr
	mng_putchunk_jsep
	mng_putchunk_loop
	mng_putchunk_magn
	mng_putchunk_mend
	mng_putchunk_mhdr
	mng_putchunk_move
	mng_putchunk_need
	mng_putchunk_ordr
	mng_putchunk_ordr_entry
	mng_putchunk_past
	mng_putchunk_past_src
	mng_putchunk_phyg
	mng_putchunk_phys
	mng_putchunk_plte
	mng_putchunk_pplt
	mng_putchunk_pplt_entry
	mng_putchunk_prom
	mng_putchunk_save
	mng_putchunk_save_entry
	mng_putchunk_sbit
	mng_putchunk_seek
	mng_putchunk_show
	mng_putchunk_splt
	mng_putchunk_srgb
	mng_putchunk_term
	mng_putchunk_text
	mng_putchunk_time
	mng_putchunk_trns
	mng_putchunk_unknown
	mng_putchunk_ztxt
	mng_putimgdata_ihdr
	mng_putimgdata_jhdr
	mng_read
	mng_read_resume
	mng_readdisplay
	mng_reset
	mng_set_bgcolor
	mng_set_bkgdstyle
	mng_set_cacheplayback
	mng_set_canvasstyle
	mng_set_dfltimggamma
	mng_set_dfltimggammaint
	mng_set_displaygamma
	mng_set_displaygammaint
	mng_set_doprogressive
	mng_set_jpeg_dctmethod
	mng_set_jpeg_maxjdat
	mng_set_jpeg_optimized
	mng_set_jpeg_progressive
	mng_set_jpeg_quality
	mng_set_jpeg_smoothing
	mng_set_maxcanvasheight
	mng_set_maxcanvassize
	mng_set_maxcanvaswidth
	mng_set_outputprofile
	mng_set_outputprofile2
	mng_set_outputsrgb
	mng_set_sectionbreaks
	mng_set_speed
	mng_set_srgb
	mng_set_srgbimplicit
	mng_set_srgbprofile
	mng_set_srgbprofile2
	mng_set_storechunks
	mng_set_suspensionmode
	mng_set_usebkgd
	mng_set_userdata
	mng_set_viewgamma
	mng_set_viewgammaint
	mng_set_zlib_level
	mng_set_zlib_maxidat
	mng_set_zlib_memlevel
	mng_set_zlib_method
	mng_set_zlib_strategy
	mng_set_zlib_windowbits
	mng_setcb_closestream
	mng_setcb_errorproc
	mng_setcb_getalphaline
	mng_setcb_getbkgdline
	mng_setcb_getcanvasline
	mng_setcb_gettickcount
	mng_setcb_memalloc
	mng_setcb_memfree
	mng_setcb_openstream
	mng_setcb_processheader
	mng_setcb_processmend
	mng_setcb_processneed
	mng_setcb_processsave
	mng_setcb_processseek
	mng_setcb_processterm
	mng_setcb_processtext
	mng_setcb_processunknown
	mng_setcb_readdata
	mng_setcb_refresh
	mng_setcb_settimer
	mng_setcb_writedata
	mng_status_creating
	mng_status_displaying
	mng_status_dynamic
	mng_status_error
	mng_status_reading
	mng_status_running
	mng_status_runningevent
	mng_status_suspendbreak
	mng_status_timerbreak
	mng_status_writing
	mng_supports_func
	mng_trapevent
	mng_updatemngheader
	mng_updatemngsimplicity
	mng_version_beta
	mng_version_dll
	mng_version_major
	mng_version_minor
	mng_version_release
	mng_version_so
	mng_version_text
	mng_write
